from MySerializer.json_utils import Json
from MySerializer.xml_utils import Xml

class MySerializer:
    @staticmethod
    def createSerializer(format):
        if format == ".json":
            return Json()

        elif format == ".xml":
            return Xml()

        else:
            raise Exception("Wrong format")    


from MySerializer.constants import PRIMITIVES,COLLECTIONS
from MySerializer.pack_utils import pack
from MySerializer.unpack_utils import unpack

class Json:
    def __init__(self):
        self.pos = 0
        self.indent = 0

    def dump(self, obj, fp):
        fp.write(self.dumps(obj))

    def dumps(self, obj):
        return self.serialize_to_str(pack(obj))

    def serialize_to_str(self, obj):
        if type(obj) in PRIMITIVES:
            return self.serialize_primitive(obj)

        elif type(obj) is dict:
            return self.serialize_dict(obj)
        
        elif type(obj) in COLLECTIONS:
            return self.serialize_collection(obj)

        else:
            raise Exception("Unknown type to serialize")

    def serialize_collection(self, obj):
        result = '\n' + ' ' * self.indent + '{\n'
        result += ' ' * self.indent + f'"type": "{type(obj).__name__}"\n'
        result += ' ' * self.indent + '"value": ['

        self.indent += 4

        for i in obj:
            result += self.serialize_to_str(pack(i)) + ','

        if len(result) > 1 and result[-1] == ',':
            result = result[:-1]

        self.indent -= 4
        result += '\n' + ' ' * self.indent + ']\n' + ' ' * self.indent + '}'

        return result

    def serialize_primitive(self, obj):
        result = '\n' + ' ' * self.indent + '{\n'
        result += ' ' * self.indent + f'"type": "{type(obj).__name__}"\n'
        result += ' ' * self.indent + '"value": '

        if obj is None:
            result += 'null'

        elif isinstance(obj, bool):
            result += 'true' if obj else 'false'

        elif isinstance(obj, (int, float)):
            result += str(obj)

        elif isinstance(obj, str):
            result += f'"{obj}"'

        result += '\n' + ' ' * self.indent + '}'

        return result

    def serialize_dict(self, obj):
        result = '\n' + ' ' * self.indent + '{\n'
        result += ' ' * self.indent + f'"type": "{type(obj).__name__}"\n'
        result += ' ' * self.indent + '"value": {'

        self.indent += 4

        for key, value in obj.items():
            result += self.serialize_to_str(pack(key)) + ': ' + self.serialize_to_str(pack(value)) + ', \n'

        if len(result) > 1 and result[-3] == ',':
            result = result[:-3]

        result += '\n' + ' ' * self.indent + '}\n'
        self.indent -= 4
        result += ' ' * self.indent + '}'

        return result
    
    def loads(self, s):
        self.pos = 0

        return unpack(self.deserialize_from_str(s))

    def load(self, file):
        return self.loads(file.read())
    
    def deserialize_from_str(self, string):
        self.pos = string.find('"type":', self.pos)

        if self.pos != -1:
            self.pos += len('"type": ')

        if self.pos >= len(string) or self.pos == -1:
            return

        if string[self.pos:self.pos + len('"int"')] == '"int"':
            self.pos += len('"int"\n')

            return self.deserialize_num(string)

        if string[self.pos:self.pos + len('"float"')] == '"float"':
            self.pos += len('"float"\n')

            return self.deserialize_num(string)

        if string[self.pos:self.pos + len('"bool"')] == '"bool"':
            self.pos += len('"bool"\n')

            return self.deserialize_bool(string)

        if string[self.pos:self.pos + len('"NoneType"')] == '"NoneType"':
            self.pos += len('"NoneType"\n')

            return self.deserialize_null(string)

        if string[self.pos:self.pos + len('"str"')] == '"str"':
            self.pos += len('"str"\n')

            return self.deserialize_str(string)

        if string[self.pos:self.pos + len('"dict"')] == '"dict"':
            self.pos += len('"dict"\n')

            return self.deserialize_dict(string)

        if string[self.pos:self.pos + len('"list"')] == '"list"' or \
                string[self.pos:self.pos + len('"tuple"')] == '"tuple"' or \
                string[self.pos:self.pos + len('"set"')] == '"set"':

            self.pos += 1
            ind_end = string.find('"', self.pos)
            s_type = string[self.pos: ind_end]
            self.pos = ind_end + 1

            return self.deserialize_collection(string, s_type)

    def deserialize_num(self, s):
        self.pos = s.find('"value": ', self.pos) + len('"value": ')
        s_pos = self.pos

        while self.pos < len(s) and\
                (s[self.pos].isdigit() or s[self.pos] == '.'):
            self.pos += 1

        num = s[s_pos:self.pos]
        self.pos = s.find('}', self.pos) + 1

        return float(num) if '.' in str(num) else int(num)

    def deserialize_bool(self, s):
        self.pos = s.find('"value": ', self.pos) + len('"value": ')

        if s[self.pos:self.pos + 4] == "true":
            self.pos = s.find('}', self.pos) + 1

            return True

        else:
            self.pos = s.find('}', self.pos) + 1

            return False

    def deserialize_null(self, s):
        self.pos = s.find('"value": ', self.pos) + len('"value": ')
        self.pos = s.find('}', self.pos) + 1

        return None

    def deserialize_str(self, s):
        self.pos = s.find('"value": ', self.pos) + len('"value": ')

        res = ""
        self.pos += 1

        while self.pos < len(s) and s[self.pos:self.pos + 1] not in '"\n':
            res += s[self.pos]
            self.pos += 1

        self.pos = s.find('}', self.pos) + 1

        return res

    def deserialize_dict(self, s):
        self.pos = s.find('"value": ', self.pos) + len('"value": ')
        res = {}

        self.pos += 1

        while self.pos < len(s) and s[self.pos] != '}':
            if s[self.pos] in (' ', ',', ':', '\n'):
                self.pos += 1
                continue

            k = self.deserialize_from_str(s)
            v = self.deserialize_from_str(s)
            res[k] = v

        self.pos = s.find('}', self.pos + 1) + 1

        return res

    def deserialize_collection(self, s, s_type):
        self.pos = s.find('"value": ', self.pos) + len('"value": ')

        res = []
        self.pos += 1

        while self.pos < len(s) and s[self.pos] != ']':
            if s[self.pos] in (' ', ',', '\n'):
                self.pos += 1
                continue

            v = self.deserialize_from_str(s)
            res.append(v)

        self.pos = s.find('}', self.pos) + 1

        if s_type == 'tuple':
            return tuple(res)

        elif s_type == 'set':
            return set(res)

        return res



import inspect
import builtins
import types
from MySerializer.constants import PRIMITIVES
from types import FunctionType, LambdaType, CodeType, CellType, GeneratorType, ModuleType

def is_function(obj):
    return inspect.isfunction(obj) or inspect.ismethod(obj) or isinstance(obj, LambdaType)


def is_iterable(obj):
    return getattr(obj, "__iter__", None) is not None

def unpack(src):
    if isinstance(src, PRIMITIVES):
        return src

    elif isinstance(src, dict):
        if "function" in src.values():
            return unpack_function(src)

        elif "object" in src.values():
            return unpack_object(src)

        elif "class" in src.values():
            return unpack_class(src)
        
        elif "code" in src.values():
            return unpack_code(src)
        
        elif "iterator" in src.values():
            return unpack_iterator(src)

        else:
            return unpack_iterable(src)

    elif is_iterable(src):
        return unpack_iterable(src)

    elif "module" == src.__class__.__name__:
        return unpack_module(src)

    else:

        raise Exception("Unknown type")

def unpack_module(obj):
    return obj

def unpack_class(obj):
    class_bases = tuple(unpack_class(base) for base in obj["__bases__"])
    class_methods = {}

    for attr, value in obj.items():
        class_methods[attr] = unpack(value)

    result = type(obj["__name__"], class_bases, class_methods)

    for key, method in class_methods.items():
        if inspect.isfunction(method):
            method.__globals__.update({result.__name__: result})

    return result

def unpack_iterator(obj):
    for i in unpack(obj["__values__"]):
        yield i

def unpack_code(obj):
    attrs = {}

    for key, value in obj.items():
        attrs[key] = unpack(value)

    result = CodeType(attrs['co_argcount'],
                     attrs['co_posonlyargcount'],
                     attrs['co_kwonlyargcount'],
                     attrs['co_nlocals'],
                     attrs['co_stacksize'],
                     attrs['co_flags'],
                     bytes(attrs['co_code']),
                     tuple(unpack(attrs['co_consts'])),
                     tuple(attrs['co_names']),
                     tuple(attrs['co_varnames']),
                     attrs['co_filename'],
                     attrs['co_name'],
                     attrs['co_firstlineno'],
                     bytes(attrs['co_lnotab']),
                     tuple(attrs['co_freevars']),
                     tuple(attrs['co_cellvars']))

    return result

def unpack_object(obj):
    obj_class = unpack(obj["__class__"])
    attrs = {}

    for key, value in obj["attr"].items():
        attrs[key] = unpack(value)

    if "property" in obj_class.__name__:
        obj_class = property
        result = property(fget=attrs["fget"],fset=attrs["fset"],fdel=attrs["fdel"])
    elif "cell" in obj_class.__name__:
        result = CellType(attrs["cell_contents"])
    else:
        result = object.__new__(obj_class)
        result.__dict__ = attrs

    return result

def unpack_function(src):
    arguments = src["__args__"]
    globs = src["__globals__"]
    globs["__builtins__"] = __builtins__

    for key in src["__globals__"]:
        if key in arguments["co_names"]:
            try:
                globs[key] = __import__(src["__globals__"][key])

            except:
                if globs[key] != src["__name__"]:
                    globs[key] = unpack(src["__globals__"][key])


    coded = CodeType(arguments['co_argcount'],
                     arguments['co_posonlyargcount'],
                     arguments['co_kwonlyargcount'],
                     arguments['co_nlocals'],
                     arguments['co_stacksize'],
                     arguments['co_flags'],
                     bytes(arguments['co_code']),
                     tuple(arguments['co_consts']),
                     tuple(arguments['co_names']),
                     tuple(arguments['co_varnames']),
                     arguments['co_filename'],
                     arguments['co_name'],
                     arguments['co_firstlineno'],
                     bytes(arguments['co_lnotab']),
                     tuple(arguments['co_freevars']),
                     tuple(arguments['co_cellvars']))

    func_result = FunctionType(coded, globs, closure = unpack(src["__closure__"]))
    func_result.__globals__.update({func_result.__name__: func_result})

    return func_result


def unpack_iterable(obj):
    if isinstance(obj, list) or isinstance(obj, tuple) or isinstance(obj, set):
        unpacked_iterable = []

        for value in obj:
            unpacked_iterable.append(unpack(value))

        if isinstance(obj, tuple):
            return tuple(unpacked_iterable)

        if isinstance(obj, set):
            return set(unpacked_iterable)

        return unpacked_iterable

    elif isinstance(obj, dict):
        unpacked_dict = {}

        for key, value in obj.items():
            unpacked_dict[key] = unpack(value)

        return unpacked_dict


def turn_obj_into_dict(obj):
    if isinstance(obj, type(None)):

        return {
            "None": "None"
        }

    elif isinstance(obj, PRIMITIVE_TYPES):

        return {
            str(type(obj)): obj
        }

    elif isinstance(obj, (list, tuple, set)):
        result = []

        for item in obj:
            result.append(turn_obj_into_dict(item))

        return {
            str(type(obj)): result
        }

    elif isinstance(obj, dict):
        result = {}

        for key, value in obj.items():
            result[key] = turn_obj_into_dict(value)

        return result

    else:
        raise Exception("Unknown type")


def restore_object_from_dict(src):
    if type(src) is dict:

        if len(src.keys()) == 1:
            key, value = list(src.items())[0]

            if key == "None":
                return None

            elif isinstance(value, PRIMITIVE_TYPES):
                return value

            elif isinstance(value, list):
                result = []

                for obj in value:
                    result.append(restore_object_from_dict(obj))

                if key == "<class 'tuple'>":
                    result = tuple(result)

                elif key == "<class 'set'>":
                    result = set(result)

                return result

        result = {}

        for key, val in src.items():
            result[key] = restore_object_from_dict(val)

        return result

    else:
        raise Exception("Object type must be dict")


import math

def fooMath(a):
    return math.sin(a)

def foo(a,b):
    return a+b

def closure(a):
    def inner(a):
        return 0
    
    return inner

def gen(a):
    yield a
    yield a
    yield a

lam = lambda x: x+1

def second(a):
    return a*a

def first(a):
    return second(2)+a

def firstIn(a):
    b = second
    return b(a)+a

def rec(a):
    if a!=0:
        return a+rec(a-1)
    else: 
        return 0
    

class A:
    a_var = 1

    def a_func(self):
        return "a"
    
    def letter(self):
        return "a"

    def __init__(self,num):
        self.num = num

    @property
    def prop_test(self):
        return self.num
    
class B(A):
    def letter(self):
        return "c"
    
    def b_func(self):
        return "b"

class D(A):
    def letter(self):
        return "d"

class C(B,D):
    pass


import inspect
import types
from MySerializer.constants import PRIMITIVES, COLLECTIONS
from types import FunctionType, BuiltinFunctionType, LambdaType, GetSetDescriptorType, MappingProxyType, \
    MethodDescriptorType, WrapperDescriptorType


def is_function(obj):
    return inspect.isfunction(obj) or inspect.ismethod(obj) or isinstance(obj, LambdaType)


def is_iterable(obj):
    return getattr(obj, "__iter__", None) is not None


def pack(obj):
    if isinstance(obj, PRIMITIVES):
        return obj
    
    elif isinstance(obj, COLLECTIONS):
        return obj

    elif is_function(obj):
        return pack_function(obj)

    elif inspect.iscode(obj):
        return pack_code(obj)

    elif inspect.isclass(obj):
        return pack_class(obj)

    elif is_iterable(obj):
        return pack_iterable(obj)

    else:
        return pack_object(obj)


def pack_function(obj, cls=None):
    result = {"__type__": "function"}

    if inspect.ismethod(obj):
        obj = obj.__func__

    result["__name__"] = obj.__name__

    globs = get_global_vars(obj, cls)
    result["__globals__"] = pack_iterable(globs)
    result["__closure__"] = pack(obj.__closure__)

    
    arguments = {}

    for (key, value) in inspect.getmembers(obj.__code__):
        if key.startswith("co_"):
            if isinstance(value, bytes):
                value = list(value)

            if is_iterable(value) and not isinstance(value, str):
                packed_vals = []

                for val in value:
                    if val is not None:
                        packed_vals.append(pack(val))

                    else:
                        packed_vals.append(None)

                arguments[key] = packed_vals

                continue

            arguments[key] = value

    result["__args__"] = arguments

    return result


def get_global_vars(func, cls):
    globs = {}

    for global_var in func.__code__.co_names:
        if global_var in func.__globals__:
            if isinstance(func.__globals__[global_var], types.ModuleType):
                globs[global_var] = func.__globals__[global_var].__name__

            elif inspect.isclass(func.__globals__[global_var]):
                if cls and func.__globals__[global_var] != cls:
                    globs[global_var] = func.__globals__[global_var]

            elif global_var != func.__code__.co_name:
                globs[global_var] = func.__globals__[global_var]

            else:
                globs[global_var] = func.__name__
    return globs


def pack_iterable(obj):

    if isinstance(obj, list) or isinstance(obj, tuple) or isinstance(obj, set) or isinstance(obj, bytes):
        packed_iterable = []

        for value in obj:
            packed_iterable.append(pack(value))

        if isinstance(obj, tuple):
            return tuple(packed_iterable)

        if isinstance(obj, set):
            return set(packed_iterable)

        return packed_iterable

    elif isinstance(obj, dict):
        packed_dict = {}

        for key, value in obj.items():
            packed_dict[key] = pack(value)

        return packed_dict
    
    else:
        result = {"__type__": "iterator"}
        values = []
        for i in obj:
            values.append(pack(i))
        
        result["__values__"] = values

        return result


def pack_code(obj):   
    result = {"__type__":"code"}

    for key, value in inspect.getmembers(obj):
        if not key.startswith("co"):
            continue

        result[key] = pack(value)

    return result


def pack_class(obj):
    result = {'__type__': 'class', '__name__': obj.__name__}

    for attr in inspect.getmembers(obj):
        if attr[0] not in (
                "__mro__", "__base__", "__basicsize__",
                "__class__", "__dictoffset__", "__name__",
                "__qualname__", "__text_signature__", "__itemsize__",
                "__flags__", "__weakrefoffset__", "__objclass__"
        ) and type(attr[1]) not in (
                WrapperDescriptorType,
                MethodDescriptorType,
                BuiltinFunctionType,
                MappingProxyType,
                GetSetDescriptorType
        ):
            attr_value = getattr(obj, attr[0])

            if is_function(attr_value):
                result[attr[0]] = pack_function(attr_value, obj)

            else:
                result[attr[0]] = pack(attr_value)

    result["__bases__"] = [pack_class(base) for base in obj.__bases__ if base != object]

    return result

def pack_object(obj):
    result = {"__type__": "object", "__class__": pack_class(obj.__class__), "attr": {}}
    if(obj.__class__.__name__ in ["property","cell"]):
        for key, value in inspect.getmembers(obj):
            if not key.startswith("__"):
                result["attr"][key] = pack(value)

    else:
        for key, value in inspect.getmembers(obj):
            if not key.startswith("__") and not is_function(value):
                result["attr"][key] = pack(value)

    return result



from MySerializer.constants import PRIMITIVES,COLLECTIONS
from MySerializer.pack_utils import pack
from MySerializer.unpack_utils import unpack


class Xml:
    def __init__(self):
        self.pos = 0
        self.indent = 0

    def dumps(self, obj):
        return self.serialize_to_str(pack(obj))

    def dump(self, obj, file):
        file.write(self.dumps(obj))

    def serialize_to_str(self, obj):
        if type(obj) in PRIMITIVES:
            return self.serialize_primitive(obj)

        elif type(obj) is dict:
            return self.serialize_dict(obj)
        
        elif type(obj) in COLLECTIONS:
            return self.serialize_collection(obj)

        else:
            raise Exception("Unknown type to serialize")


    def serialize_dict(self, obj):
        result = f'<"{type(obj).__name__}">\n'
        self.indent += 4

        for key, value in obj.items():
            result += ' ' * self.indent + f'<{self.serialize_to_str(pack(key))}>\n'
            self.indent += 4
            result += ' ' * self.indent + f'{self.serialize_to_str(pack(value))}' + '\n'
            self.indent -= 4
            result += ' ' * self.indent + f'</{type(key).__name__}>' + '\n'

        if len(result) > 1 and result[-3] == ',':
            result = result[:-3]

        self.indent -= 4
        result += ' ' * self.indent + f'</"{type(obj).__name__}">'

        return result

    def serialize_primitive(self, obj):
        result = ''

        if obj is None:
            result += f'<"{type(obj).__name__}">' + 'null' + f'</"{type(obj).__name__}">'

        elif isinstance(obj, bool):
            result += f'<"{type(obj).__name__}">' + 'true' + f'</"{type(obj).__name__}">' if obj\
                else f'<"{type(obj).__name__}">' + 'false' + f'</"{type(obj).__name__}">'

        elif isinstance(obj, (int, float)):
            result += f'<"{type(obj).__name__}">' + str(obj) + f'</"{type(obj).__name__}">'

        elif isinstance(obj, str):
            result += f'<"{type(obj).__name__}">' + f'{obj}' + f'</"{type(obj).__name__}">'

        return result

    def serialize_collection(self, obj):
        result = f'<"{type(obj).__name__}">\n'
        self.indent += 4

        for i in obj:
                result += ' ' * self.indent + f'{self.serialize_to_str(pack(i))}\n'

        self.indent -= 4
        result += ' ' * self.indent + f'</"{type(obj).__name__}">'

        return result

    def loads(self, s):
        self.pos = 0
        return unpack(self.deserialize_from_str(s))

    def load(self, file):
        return self.loads(file.read())

    def deserialize_from_str(self, string):
        self.pos = string.find('<"', self.pos)

        if self.pos != -1:
            self.pos += len('<')  

        if self.pos >= len(string) or self.pos == -1:
            return None

        if string[self.pos:self.pos + len('"int"')] == '"int"':
            self.pos += len('"int"')

            return self.deserialize_num(string)

        if string[self.pos:self.pos + len('"float"')] == '"float"':
            self.pos += len('"float"')

            return self.deserialize_num(string)

        if string[self.pos:self.pos + len('"bool"')] == '"bool"':
            self.pos += len('"bool"')

            return self.deserialize_bool(string)

        if string[self.pos:self.pos + len('"NoneType"')] == '"NoneType"':
            self.pos += len('"NoneType"')

            return self._deserialize_null(string)

        if string[self.pos:self.pos + len('"str"')] == '"str"':
            self.pos += len('"str"')

            return self.deserialize_str(string)

        if string[self.pos:self.pos + len('"dict"')] == '"dict"':
            self.pos += len('"dict"')
            return self.deserialize_dict(string)

        if string[self.pos:self.pos + len('"list"')] == '"list"' or \
                string[self.pos:self.pos + len('"tuple"')] == '"tuple"' or \
                string[self.pos:self.pos + len('"set"')] == '"set"':

            self.pos += 1
            ind_end = string.find('"', self.pos)
            s_type = string[self.pos: ind_end]
            self.pos = ind_end + 2

            return self.deserialize_collection(string, s_type)

    def deserialize_num(self, s):
        self.pos = s.find('>', self.pos) + len('>')
        s_pos = self.pos

        while self.pos < len(s) and\
                ((s[self.pos].isdigit() or s[self.pos] == '.') and s[self.pos] != '<'):
            self.pos += 1

        num = s[s_pos:self.pos]
        self.pos = s.find('>', self.pos) + 1

        return float(num) if '.' in str(num) else int(num)

    def deserialize_collection(self, s, s_type):
        res = []
        self.pos = s.find('<', self.pos + 1)
        while self.pos < len(s) and\
                s[self.pos: self.pos + len(s_type) + 5] != f'</"{s_type}">':

            v = self.deserialize_from_str(s)
            res.append(v)
            self.pos = s.find('<', self.pos)

        if s_type == 'tuple':
            self.pos += 1
            return tuple(res)

        elif s_type == 'set':
            self.pos += 1
            return set(res)
        else:
            self.pos += 1
            return res

    def deserialize_dict(self, s):
        res = {}

        while self.pos < len(s) and s[self.pos:self.pos+len('</"dict">')] != '</"dict">':
            self.pos = s.find('<', self.pos)

            while s[self.pos+1] == '/' and s[self.pos:self.pos+9] != '</"dict">':
                self.pos = s.find('<', self.pos+1)
            if s[self.pos+1] == '/' and s[self.pos:self.pos+9] == '</"dict">':
                self.pos = s.find('<', self.pos + 1)
                return res
            k = self.deserialize_from_str(s)
            v = self.deserialize_from_str(s)

            res[k] = v

            if self.pos == -1:
                return res

        return res

    def deserialize_str(self, s):
        res = ''
        self.pos = s.find('>', self.pos) + 1

        while self.pos < len(s) and s[self.pos] != '<':
            res += s[self.pos]
            self.pos += 1

        self.pos = s.find('>', self.pos)
        return res

    def deserialize_bool(self, s):
        self.pos = s.find('>', self.pos) + len('>')

        if s[self.pos:self.pos + 4] == "true":
            self.pos = s.find('>', self.pos) + 1
            return True

        else:
            self.pos = s.find('>', self.pos) + 1
            return False

    def _deserialize_null(self, s):
        self.pos = s.find('>', self.pos) + len('>')
        self.pos = s.find('>', self.pos) + 1

        return None


from argparse import ArgumentParser
from MySerializer.MySerializer import MySerializer

def main():
    try:
        parse = ArgumentParser()
        parse.add_argument('file_from',type=str, help='file from which you load data')    
        parse.add_argument('file_to',type=str, help='file to which you save serialized data')
        parse.add_argument('format_from',type=str, help='format from which you deserialize data, can be any of json/xml')
        parse.add_argument('format_to',type=str, help='format to which you serialize data, can be any of json/xml')

        args =  parse.parse_args()

        if args.file_from == args.file_to:
            print("File from and file to are the same. Please check your data and try again")
            exit()

        if args.format_from == args.format_to:
            print("Format from and format to are the same. Please check your data and try again")
            exit()

        from_serializer = MySerializer.createSerializer(args.format_from)
        to_serializer = MySerializer.createSerializer(args.format_to)

        with open(args.file_from) as file:
            obj = from_serializer.load(file)
            with open(args.file_to, "w") as output_file:
                to_serializer.dump(obj, output_file)
    except Exception as e:
        print(e)


from setuptools import setup, find_packages


setup(
    name="denvilk-serializer",
    version="1.0.0",
    description="JSON and XLM serializer",
    author="Vladimir Velikovich",
    author_email="vvelikovich@gmail.com",
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        'console_scripts': [ 
            'serializer = MySerializer.serializer:main' 
        ],
    },
)